// src/mini-app-messages.ts
import { z } from "zod";
var createCastRequestSchemaLegacy = z.object({
  type: z.literal("createCast"),
  data: z.object({
    cast: z.object({
      parent: z.string().optional(),
      text: z.string(),
      embeds: z.array(z.string().min(1).url()).min(1)
    })
  })
});
var createCastRequestSchema = z.object({
  jsonrpc: z.literal("2.0"),
  id: z.union([z.string(), z.number(), z.null()]),
  method: z.literal("fc_createCast"),
  params: z.object({
    cast: z.object({
      parent: z.string().optional(),
      text: z.string(),
      embeds: z.array(z.string().min(1).url()).min(1)
    })
  })
});
var ethSendTransactionActionSchema = z.object({
  chainId: z.string(),
  method: z.literal("eth_sendTransaction"),
  attribution: z.boolean().optional(),
  params: z.object({
    abi: z.custom(),
    to: z.custom(
      (val) => typeof val === "string" && val.startsWith("0x")
    ),
    value: z.string().optional(),
    data: z.custom((val) => typeof val === "string" && val.startsWith("0x")).optional()
  })
});
var ethSignTypedDataV4ActionSchema = z.object({
  chainId: z.string(),
  method: z.literal("eth_signTypedData_v4"),
  params: z.object({
    domain: z.custom(),
    types: z.custom((value) => {
      const result = z.record(z.unknown()).safeParse(value);
      return result.success;
    }),
    primaryType: z.string(),
    message: z.record(z.unknown())
  })
});
var walletActionRequestSchema = z.object({
  jsonrpc: z.literal("2.0"),
  id: z.string(),
  method: z.literal("fc_requestWalletAction"),
  params: z.object({
    action: z.union([
      ethSendTransactionActionSchema,
      ethSignTypedDataV4ActionSchema
    ])
  })
});
var miniAppMessageSchema = z.union([
  createCastRequestSchemaLegacy,
  walletActionRequestSchema,
  createCastRequestSchema
]);

export {
  miniAppMessageSchema
};
